function Parameters(varargin)
% Parameters(varargin)
% Usage: This function sets all the parameters needed for the GP.
% varargin = {comm}
% The comm structure is the communication factor between the screens.
% comm.pos = The last figure position. The current figure will take its place.
% comm.dirname = The name of the work directory.
% comm.name = the project's name.
% comm.movie = Whether the "save movie" check box was checked (project).
% comm.plot = Whether to plot axexs on startup (LoadData)
% comm.UseFilter = Whether the filter was used the last time (LoadData).
% comm.LoadParamaters = Whether the user has changed any of the paramaters (Parameters).
% comm.path1 = The Path to DataSet1.
% comm.path2 = The Path to DataSet2.
% comm.spath1 = The short Path name to DataSet1.
% comm.spath2 = The short Path name to DataSet2.
% NOTE:
% comm.dirname is the WORKING dir name.
% comm.path# is the DATA dir name.
% Credits: Alex Tesler, Yehu David-Horev, Alon Dana, Maor Asher.
% 2.y variation by yonatan vaisberg
% Modified by: Arthur Doroshev
%--------------------------------------------------------------------------
global par comm 
if nargin==0
    % Set Default values:
    comm.pos=[240,227,800,550];
    comm.dirname='0';
    comm.existingfile='';
    comm.name='';
    comm.ramp(1:2,1:5)=-1;
    comm.movie=0;
    comm.plot=0;
    comm.UseFilter=0;
    comm.LoadParamaters=0;
    comm.path1='';
    comm.path2='';
    comm.spath1='';
    comm.spath2='';
    par.stop='';
else
    comm=varargin{1};
end

if strcmp(comm.check,'')
    try
        load([comm.dirname,'\','par.mat']); 
    catch
        par=ParDefaults();
    end
end

save([comm.dirname,'\','par.mat'],'-struct','par');
comm.pos=[240,227,800,600];
screen4=figure('Position',comm.pos,'Color','White','MenuBar','none','NumberTitle','off','Name','Impedance Spectroscopy Genetic Programming','Resize','off','Tag','screen4');
movegui(screen4,'center');
handles=guihandles;
guidata(screen4, handles);

%------------ Buttons: ------------%
% Close:
handles.close=uicontrol('Style','pushbutton','Units','pixels','Position',[450,10,70,25],'FontSize',8,'FontWeight','bold','String','Close');

% Help:
handles.help=uicontrol('Style','pushbutton','Units','pixels','Position',[540,10,70,25],'FontSize',8,'FontWeight','bold','String','Help');

% Next (RUN):
handles.next=uicontrol('Style','pushbutton','Units','pixels','Position',[720,10,70,25],'FontWeight','bold','FontSize',12,'Enable','on','String','Run');

% Back:
handles.back=uicontrol('Style','pushbutton','Units','pixels','Position',[630,10,70,25],'FontSize',8,'FontWeight','bold','String','<  Back');

% Header:
axes('Units','pixels','Position',[316.5,576,164,22],'CreateFcn',{@header,handles});

% slogo:
%axes('Units','pixels','Position',[0 563 118 37],'CreateFcn',{@slogo,handles});

% text: Project Name
uicontrol('Style','text','Units','pixels','Position',[645,580,45,20],'BackgroundColor','White','FontSize',9,'HorizontalAlignment','left','String','Project: ');
n=uicontrol('Style','text','Units','pixels','Position',[690,580,110,20],'BackgroundColor','White','FontSize',9,'FontWeight','bold','HorizontalAlignment','left','String',comm.name);

if comm.dirname=='0'
    set(n,'String','No Project Loaded');
    set(n,'ForegroundColor','red');
    set(n,'FontAngle','italic');
    set(n,'FontWeight','normal');
end

% text: "The Filter is not to scale in the Y axis."
handles.NotScale=uicontrol('Style','text','Units','pixels','Position',[380,35,180,20],'Visible','off','BackgroundColor','White','ForegroundColor','black','FontSize',8,'HorizontalAlignment','left','String','(The Filter is not to scale in the Y axis.)');

%------------ Axes: ------------%
handles.ax1=axes('Units','pixels','Position',[415,380,365,160]);
handles.ax2=axes('Units','pixels','Position',[415,135,365,160]);

%------------ Population & Propability Panel: ------------%
proppnl=uipanel('Title','Population & Propability','FontSize',10,'Parent',handles.screen4,'Units','pixels','FontSize',10,'Position',[10,388,350,175],'BackgroundColor','White');

% text: "Number of runs:"
uicontrol('Style','text','Units','pixels','Parent',proppnl,'Position',[5,130,120,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Number of runs:');
handles.runs=uicontrol('Style','edit','Units','pixels','Parent',proppnl,'Position',[110,130,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.runs));

% text: "Population size:"
uicontrol('Style','text','Units','pixels','Parent',proppnl,'Position',[5,105,120,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Population size:');
handles.population=uicontrol('Style','edit','Units','pixels','Parent',proppnl,'Position',[110,105,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.population));

% text: "Remove peak propability:"
uicontrol('Style','text','Units','pixels','Parent',proppnl,'Position',[5,80,120,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Remove peak:');
handles.remove=uicontrol('Style','edit','Units','pixels','Parent',proppnl,'Position',[110,80,30,20],'BackgroundColor','White','Enable','inactive','FontSize',10,'HorizontalAlignment','center','String',num2str(par.remove));

% text: "Mutation propability:"
uicontrol('Style','text','Units','pixels','Parent',proppnl,'Position',[163,130,140,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Mutation propability:');
handles.mutation=uicontrol('Style','edit','Units','pixels','Parent',proppnl,'Position',[312,130,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.mutation));

% text: "Add peak propability:"
uicontrol('Style','text','Units','pixels','Parent',proppnl,'Position',[163,105,140,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Add peak propability:');
handles.addpick=uicontrol('Style','edit','Units','pixels','Parent',proppnl,'Position',[312,105,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.addpick));

% text: "Duplication factor:"
uicontrol('Style','text','Units','pixels','Parent',proppnl,'Position',[163,80,140,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Duplication factor:');
handles.duplicate=uicontrol('Style','edit','Units','pixels','Parent',proppnl,'Position',[312,80,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.duplicate));

% text: "Maximum descendants per individual function:"
uicontrol('Style','text','Units','pixels','Parent',proppnl,'Position',[5,55,270,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Maximum descendants per individual function:');
handles.max=uicontrol('Style','edit','Units','pixels','Parent',proppnl,'Position',[312,55,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.max));

% text: "Stop Criterion - Generations Without Improvement:"
uicontrol('Style','text','Units','pixels','Parent',proppnl,'Position',[5,28,330,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Stop Criterion - Generations Without Improvement:');
handles.stopCr=uicontrol('Style','edit','Units','pixels','Parent',proppnl,'Position',[312,28,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.stop));

%------------ Initial Population \ Mutation Weight Panel: ------------%
inimutpnl=uipanel('Title','Initial Population \ Mutation Weight','FontSize',10,'Parent',handles.screen4,'Units','pixels','Position',[10,128,350,250],'BackgroundColor','White');

% More peaks:
handles.Picks=uicontrol('Parent',inimutpnl,'Style','pushbutton','Units','pixels','Position',[10,10,70,25],'FontSize',8,'FontWeight','bold','String','More Peaks');

% Out Of Range Peaks:
handles.outpicks=uicontrol('Parent',inimutpnl,'Style','pushbutton','Units','pixels','Position',[90,10,120,25],'FontSize',8,'FontWeight','bold','String','Out of Range Peaks');

% Forced Peak:
handles.FPick=uicontrol('Parent',inimutpnl,'Style','pushbutton','Units','pixels','Position',[220,10,70,25],'FontSize',8,'FontWeight','bold','String','Forced Peak');

% text: "initial":
handles.gauss.txt=uicontrol('Style','text','Units','pixels','Position',[135,210,50,20],'BackgroundColor','White','FontSize',10,'FontWeight','bold','HorizontalAlignment','center','Parent',inimutpnl,'String','initial');

% text: "mutation":
handles.gauss.txt=uicontrol('Style','text','Units','pixels','Position',[180,210,70,20],'BackgroundColor','White','FontSize',10,'FontWeight','bold','HorizontalAlignment','center','Parent',inimutpnl,'String','mutation');

% text: "lb":
handles.gauss.txt=uicontrol('Style','text','Units','pixels','Position',[260,210,30,20],'BackgroundColor','White','FontSize',10,'FontWeight','bold','HorizontalAlignment','center','Parent',inimutpnl,'String','lb');

% text: "ub":
handles.gauss.txt=uicontrol('Style','text','Units','pixels','Position',[300,210,30,20],'BackgroundColor','White','FontSize',10,'FontWeight','bold','HorizontalAlignment','center','Parent',inimutpnl,'String','ub');

% text: "Pseudo Delta"
handles.delta.txt=uicontrol('Style','text','Units','pixels','Parent',inimutpnl,'Position',[5,185,133,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String','Pseudo Delta:');
handles.delta.initial=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[152,185,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.delta.initial,'String','');
handles.delta.mutation=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[205,185,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.delta.mutation,'String','');
handles.delta.lb=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[260,185,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.delta.lb));
handles.delta.ub=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[300,185,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.delta.ub));

if get(handles.delta.initial,'Value')+get(handles.delta.mutation,'Value')>0
    set(handles.delta.txt,'FontWeight','bold');
    set(handles.delta.lb,'Enable','on');
    set(handles.delta.ub,'Enable','on');
end

% text: "Gaussian"
handles.gauss.txt=uicontrol('Style','text','Units','pixels','Parent',inimutpnl,'Position',[5,160,133,20],'BackgroundColor','White','FontSize',10,'FontWeight','normal','HorizontalAlignment','center','Parent',inimutpnl,'String','Gauss:');
handles.gauss.initial=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[152,160,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.gauss.initial,'String','');
handles.gauss.mutation=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[205,160,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.gauss.mutation,'String','');
handles.gauss.lb=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[260,160,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.gauss.lb));
handles.gauss.ub=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[300,160,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.gauss.ub));

if get(handles.gauss.initial,'Value')+get(handles.gauss.mutation,'Value')>0
    set(handles.gauss.txt,'FontWeight','bold');
    set(handles.gauss.lb,'Enable','on');
    set(handles.gauss.ub,'Enable','on');
end

% text: "Lorentzian"
handles.lorenz.txt=uicontrol('Style','text','Units','pixels','Parent',inimutpnl,'Parent',inimutpnl,'Position',[5,135,133,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String','Lorentz:');
handles.lorenz.initial=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[152,135,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.lorenz.initial,'String','');
handles.lorenz.mutation=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[205,135,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.lorenz.mutation,'String','');
handles.lorenz.lb=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[260,135,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.lorenz.lb));
handles.lorenz.ub=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[300,135,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.lorenz.ub));

if get(handles.lorenz.initial,'Value')+get(handles.lorenz.mutation,'Value')>0 
    set(handles.lorenz.txt,'FontWeight','bold');
    set(handles.lorenz.lb,'Enable','on');
    set(handles.lorenz.ub,'Enable','on');
end

% text: "Sech"
handles.sech.txt=uicontrol('Style','text','Units','pixels','Parent',inimutpnl,'Position',[5,110,133,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String','Sech:');
handles.sech.initial=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[152,110,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.sech.initial,'String','');
handles.sech.mutation=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[205,110,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.sech.mutation,'String','');
handles.sech.lb=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[260,110,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.sech.lb));
handles.sech.ub=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[300,110,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.sech.ub));

if get(handles.sech.initial,'Value')+get(handles.sech.mutation,'Value')>0
    set(handles.sech.txt,'FontWeight','bold');
    set(handles.sech.lb,'Enable','on');
    set(handles.sech.ub,'Enable','on');
end

% text: "Cole Cole"
handles.cole.txt=uicontrol('Style','text','Units','pixels','Parent',inimutpnl,'Position',[5,85,133,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String','Cole Cole:');
handles.cole.initial=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[152,85,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.cole.initial,'String','');
handles.cole.mutation=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[205,85,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.cole.mutation,'String','');
handles.cole.lb=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[260,85,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.cole.lb));
handles.cole.ub=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[300,85,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.cole.ub));

if get(handles.cole.initial,'Value')+get(handles.cole.mutation,'Value')>0
    set(handles.cole.txt,'FontWeight','bold');
    set(handles.cole.lb,'Enable','on');
    set(handles.cole.ub,'Enable','on');
end

% text: "Pearson"
handles.pearson.txt=uicontrol('Style','text','Units','pixels','Parent',inimutpnl,'Position',[0,60,153,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String','Pearson:');
handles.pearson.initial=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[152,60,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.pearson.initial,'String','');
handles.pearson.mutation=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[205,60,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.pearson.mutation,'String','');
handles.pearson.lb=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[260,60,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.pearson.lb));
handles.pearson.ub=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[300,60,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.pearson.ub));

if get(handles.pearson.initial,'Value')+get(handles.pearson.mutation,'Value')>0
    set(handles.pearson.txt,'FontWeight','bold');
    set(handles.pearson.lb,'Enable','on');
    set(handles.pearson.ub,'Enable','on');
end

% text: "Losev"
handles.losev.txt=uicontrol('Style','text','Units','pixels','Parent',inimutpnl,'Position',[5,35,133,20],'BackgroundColor','White','FontSize',8,'HorizontalAlignment','center','String','Losev:');
handles.losev.initial=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[152,35,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.losev.initial,'String','');
handles.losev.mutation=uicontrol('Style','checkbox','Units','pixels','Parent',inimutpnl,'Position',[205,35,80,20],'BackgroundColor','White','FontSize',10,'Max',1,'Min',0,'Value',par.losev.mutation,'String','');
handles.losev.lb=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[260,35,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.losev.lb));
handles.losev.ub=uicontrol('Style','edit','Units','pixels','Parent',inimutpnl,'Enable','off','Position',[300,35,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.losev.ub));

if get(handles.losev.initial,'Value')+get(handles.losev.mutation,'Value')>0 
    set(handles.losev.txt,'FontWeight','bold');
    set(handles.losev.lb,'Enable','on');
    set(handles.losev.ub,'Enable','on');
end

set(handles.delta.initial,'CallBack',{@delta_check,handles});
set(handles.delta.mutation,'CallBack',{@delta_check,handles});
set(handles.gauss.initial,'CallBack',{@gauss_check,handles});
set(handles.gauss.mutation,'CallBack',{@gauss_check,handles});
set(handles.lorenz.initial,'CallBack',{@lorenz_check,handles});
set(handles.lorenz.mutation,'CallBack',{@lorenz_check,handles});
set(handles.sech.initial,'CallBack',{@sech_check,handles});
set(handles.sech.mutation,'CallBack',{@sech_check,handles});
set(handles.cole.initial,'CallBack',{@cole_check,handles});
set(handles.cole.mutation,'CallBack',{@cole_check,handles});
set(handles.pearson.initial,'CallBack',{@pearson_check,handles});
set(handles.pearson.mutation,'CallBack',{@pearson_check,handles});
set(handles.losev.initial,'CallBack',{@losev_check,handles}); 
set(handles.losev.mutation,'CallBack',{@losev_check,handles});
 
%------------ Advanced Parameters Panel: ------------%
advpnl=uipanel('Title','Advanced Parameters','FontSize',10,'Parent',handles.screen4,'Units','pixels','Position',[10,10,350,110],'BackgroundColor','White');

% "Normalization Factor:"
uicontrol('Style','text','Units','pixels','Parent',advpnl,'Position',[5,40,150,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Normalization Factor:');
handles.fitnessgauss=uicontrol('Style','edit','Units','pixels','Position',[165,40,35,20],'Parent',advpnl,'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.fitnessgauss));

% "Expected Number of Peaks:"
uicontrol('Style','text','Units','pixels','Parent',advpnl,'Position',[5,65,160,20],'BackgroundColor','White','FontSize',9,'HorizontalAlignment','left','String','Expected No. of Peaks:');
handles.beta=uicontrol('Style','edit','Units','pixels','Parent',advpnl,'Position',[165,65,35,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.beta));

% "Point Diff Factor:"
uicontrol('Style','text','Units','pixels','Parent',advpnl,'Position',[5,15,150,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Point Diff. Factor:');
handles.tau_diff=uicontrol('Style','edit','Units','pixels','Parent',advpnl,'Position',[165,15,35,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.tau_diff));

% "Width factor:"
uicontrol('Style','text','Units','pixels','Parent',advpnl,'Position',[210,65,150,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Width factor:');
handles.width=uicontrol('Style','edit','Units','pixels','Parent',advpnl,'Position',[310,65,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.sigmamax));

% "Alpha:"
uicontrol('Style','text','Units','pixels','Parent',advpnl,'Position',[210,40,150,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','left','String','Alpha:');
handles.alpha=uicontrol('Style','edit','Units','pixels','Parent',advpnl,'Position',[310,40,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',num2str(par.alpha));

% "Fitness & Complexity" Button:
uicontrol('Style','pushbutton','Units','pixels','Parent',advpnl,'FontWeight','bold','FontSize',8,'Position',[210,7,100,25],'CallBack',{@Fitness,handles},'String','Complexity help');

%%% TIMER:
% set timer to update regularly the Remove Peak Propability based on the mutation propability and the add Peak propability:
handles.t=timer('TimerFcn',{@RemovePick,handles},'Period',1.0,'ExecutionMode','fixedDelay','ObjectVisibility','off');
start(handles.t)
set(screen4,'DeleteFcn',{@DeleteFunction,handles}');

if comm.plot==1
    CreateAxes(handles,comm)
end

comm.plot=1;

% Define Buttons' Callbacks:
set(handles.close,'CallBack',{@PBClose,handles,comm});
set(handles.help,'CallBack',{@PBHelp,handles});
set(handles.next,'CallBack',{@PBNext,handles,comm});
set(handles.back,'CallBack',{@PBBack,handles,comm});
set(handles.Picks,'CallBack',{@PBPicks,handles,comm});
set(handles.FPick,'CallBack',{@PBFPick,handles,comm});
set(handles.outpicks,'CallBack',{@PBoutpciks,handles,comm});


if comm.check==1
    err=savepar(handles,comm.dirname);
    if err==1
        return
    end

    comm.pos=get(handles.screen4,'Position');
    comm.LoadParamaters=1;
    
    if~isfolder(comm.dirname)
        errordlg('The directory specified does not exist.','Incorrect File Path','modal');
        return
    end
    save([comm.dirname,'\','comm.mat'],'comm');
    stop(handles.t)
    delete(handles.screen4)
    Run(comm)
end

%------------ Functions: ------------%
% Create
%function CreateFunction()
%clc

% Delete:
function DeleteFunction(hObject,eventdata,handles) 
stop(handles.t)

% Create Axes:
function CreateAxes(handles,comm) 
% load graphs:
data=load([comm.dirname,'\','data.gpdat'],'-ascii');
data_val=load([comm.dirname,'\','data_val.gpdat'],'-ascii');
u1=(2.*pi.*data(:,1));
u2=(2.*pi.*data_val(:,1));

semilogx(handles.ax1,u1,data(:,2),u2,data_val(:,2))
xlabel(handles.ax1,'\omega','FontWeight','bold','FontUnits','pixels','FontSize',14)
ylabel(handles.ax1,'Z''','FontWeight','bold','FontUnits','pixels','FontSize',14)
legend(handles.ax1,'set1','set2','northwest')
title(handles.ax1,'Re vs. \omega','FontWeight','bold','FontUnits','pixels','FontSize',16)

semilogx(handles.ax2,u1,-data(:,3),u2,-data_val(:,3))
legend(handles.ax2,'set1','set2','northwest')

if comm.UseFilter==1
    omega=load([comm.dirname,'\','omega.gpdat'],'-ascii');
    w0=omega(1);
    w1=omega(2);
    filter=(1+exp(-5.*(log10(u1)-log10(w0))./((log10(w1)-log10(w0))))).^(-1);
    minumum=min(min(data(:,3)),min(data_val(:,3)));

    semilogx(handles.ax2,u1,-data(:,3),u2,-data_val(:,3),u1,-filter*minumum)
    legend(handles.ax2,'set1','set2','filter','northwest')
    set(handles.NotScale,'Visible','on');
    grid on
else
    set(handles.NotScale,'Visible','off');
end

xlabel(handles.ax2,'\omega','FontWeight','bold','FontUnits','pixels','FontSize',14)
ylabel(handles.ax2,'-Z"','FontWeight','bold','FontUnits','pixels','FontSize',14)
title(handles.ax2,'Im vs. \omega','FontWeight','bold','FontUnits','pixels','FontSize',16)

% Close Button:
function PBClose(hObject,eventdata,handles,comm) 
button=questdlg('Would you like to save the parameters before closing ISGP?','Close Confirmation Dialog','Yes','No','Cancel','Yes');
if strcmp(button,'Yes')
    err=savepar(handles,comm.dirname);
    if err==1
        return
    end
end
if strcmp(button,'Cancel')
    return
end
if ~isfolder(comm.dirname)
    h=errordlg('The directory specified does not exist.','Incorrect File Path','modal');
    uiwait(h)
else
    save([comm.dirname,'\','comm.mat'],'comm');
end
stop(handles.t);
delete(handles.screen4)

% Help Button:
function PBHelp(hObject,eventdata,handles)  
web http://electroceramics.technion.ac.il/isgp/help.html

% Next Button:
function PBNext(hObject,eventdata,handles,comm) 
err=savepar(handles,comm.dirname);
if err==1
    return
end
comm.pos=get(handles.screen4,'Position');
comm.LoadParamaters=1;

if ~isfolder(comm.dirname)
    errordlg('The directory specified does not exist.','Incorrect File Path','modal');
    return
end
b=getframe(gcf);
imwrite(b.cdata,[comm.dirname,'\parameters_window.jpg'])
save([comm.dirname,'\','comm.mat'],'comm');
stop(handles.t)
delete(handles.screen4)
Run(comm)

% Back Button:
function PBBack(hObject,eventdata,handles,comm) 
err=savepar(handles,comm.dirname);
if err==1
    return
end
comm.pos=get(handles.screen4,'Position');
comm.LoadParamaters=1;

if~isfolder(comm.dirname)
    errordlg('The directory specified does not exist.','Incorrect File Path','modal');
    return
end
save([comm.dirname,'\','comm.mat'],'comm');
stop(handles.t)
LoadData(comm)
delete(handles.screen4)

% More Peaks Button:
function PBPicks(~,~,~,comm)
MorePick(comm)

% Forced Peak Button:
function PBFPick(~,~,~,comm)
ForcedPick(comm)

%OutPeaks Button:
function PBoutpciks(~,~,~,comm)
Outpicks(comm)

% Save par:
function err=savepar(handles,dirname)
global comm

par=load([comm.dirname,'\','par.mat']);
err=0;

par.runs=str2double(get(handles.runs,'string'));
if isnan(par.runs) || isinf(par.runs)
    errordlg('The number of runs must be a number.','error','modal');
    err=1;
    return
end

if par.runs<1
    errordlg('The number of runs is too small','error','modal');
    err=1;
    return
end

par.population=str2double(get(handles.population,'string'));
if isnan(par.population) || isinf(par.population)
    errordlg('The Population must be a number.','error','modal');
    err=1;
    return
end

if par.population<1
    errordlg('The Population size is too small','error','modal');
    err=1;
    return
end

par.max=str2double(get(handles.max,'string'));
if isnan(par.max) || isinf(par.max)
    errordlg('Maximum descendants must be a number.','error','modal');
    err=1;
    return
end

par.stop=str2double(get(handles.stopCr,'string'));
if isnan(par.stop) || isinf(par.stop)
    errordlg('The stop criterion must be a number.','error','modal');
    err=1;
    return
end

if par.stop<1
    errordlg('The stop criterion must be larger than 1','error','modal');
    err=1;
    return
end

div=0;
par.mutation=str2double(get(handles.mutation,'string'));
if isnan(par.mutation) || isinf(par.mutation)
    errordlg('The Mutation propability must be a number.','error','modal');
    err=1;
    return
end

if par.mutation<0
    errordlg('The Mutation propability is too small. It must be a fraction.','error','modal');
    err=1;
    return
end

if par.mutation>1
    par.mutation=par.mutation/100;
    div=1;
end

if par.mutation>1
    errordlg('The Mutation propability is too big. It must be a fraction.','error','modal');
    err=1;
    return
end

if div==1
    set(handles.mutation,'string',par.mutation);
    h=msgbox('The Mutation propability should be a fraction. The ISGP had divided it by 100.','Divation message','help') ;
    uiwait(h)
    err=1;
end

div=0;
par.addpick=str2double(get(handles.addpick,'string'));
if isnan(par.addpick) || isinf(par.addpick)
    errordlg('The Add Peak propability must be a number.','error','modal');
    err=1;
    return
end

if par.addpick<0
    errordlg('The Add Peak propability is too small. It must be a fraction.','error','modal');
    err=1;
    return
end

if par.addpick>1
    par.addpick=par.addpick/100;
    div=1;
end

if par.addpick>(1-par.mutation)
    errordlg('The Add Peak or the Mutation propabilities are too big. Together they must be lower than 1.','error','modal');
    err=1;
    return
end

if div==1
    set(handles.addpick,'string',par.addpick);
    h=msgbox('The Add Peak propability should be a fraction. The ISGP had divided it by 100.','Divation message','help') ;
    uiwait(h)
    err=1;
end

par.remove=1-par.mutation-par.addpick;

par.duplicate=str2double(get(handles.duplicate,'string'));
if isnan(par.duplicate) || isinf(par.duplicate) || par.duplicate<0
    errordlg('The duplication factor must be a positive integer ','error','modal');
    err=1;
    return
end

par.delta.initial=get(handles.delta.initial,'Value');
par.gauss.initial=get(handles.gauss.initial,'Value');
par.lorenz.initial=get(handles.lorenz.initial,'Value');
par.sech.initial=get(handles.sech.initial,'Value');
par.cole.initial=get(handles.cole.initial,'Value');
par.pearson.initial=get(handles.pearson.initial,'Value');
par.losev.initial=get(handles.losev.initial,'Value');

par.delta.mutation=get(handles.delta.mutation,'Value');
par.gauss.mutation=get(handles.gauss.mutation,'Value');
par.lorenz.mutation=get(handles.lorenz.mutation,'Value');
par.sech.mutation=get(handles.sech.mutation,'Value');
par.cole.mutation=get(handles.cole.mutation,'Value');
par.pearson.mutation=get(handles.pearson.mutation,'Value');
par.losev.mutation=get(handles.losev.mutation,'Value');

par.delta.lb=str2double(get(handles.delta.lb,'String'));
par.gauss.lb=str2double(get(handles.gauss.lb,'String'));
par.lorenz.lb=str2double(get(handles.lorenz.lb,'String'));
par.sech.lb=str2double(get(handles.sech.lb,'String'));
par.cole.lb=str2double(get(handles.cole.lb,'String'));
par.pearson.lb=str2double(get(handles.pearson.lb,'String'));
par.losev.lb=str2double(get(handles.losev.lb,'String'));

par.delta.ub=str2double(get(handles.delta.ub,'String'));
par.gauss.ub=str2double(get(handles.gauss.ub,'String'));
par.lorenz.ub=str2double(get(handles.lorenz.ub,'String'));
par.sech.ub=str2double(get(handles.sech.ub,'String'));
par.cole.ub=str2double(get(handles.cole.ub,'String'));
par.pearson.ub=str2double(get(handles.pearson.ub,'String'));
par.losev.ub=str2double(get(handles.losev.ub,'String'));

if isnan(par.delta.lb) || isinf(par.delta.lb)
    errordlg('Pseudo Delta lower bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.delta.ub) || isinf(par.delta.ub)
    errordlg('Pseudo Delta upper bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.gauss.lb) || isinf(par.gauss.lb)
    errordlg('Gauss lower bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.gauss.ub) || isinf(par.gauss.ub)
    errordlg('Gauss upper bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.lorenz.lb) || isinf(par.lorenz.lb)
    errordlg('Lorentz lower bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.lorenz.ub) || isinf(par.lorenz.ub)
    errordlg('Lorentz upper bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.sech.lb) || isinf(par.sech.lb)
    errordlg('Sech bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.sech.ub) || isinf(par.sech.ub)
    errordlg('Sech upper bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.cole.lb) || isinf(par.cole.lb)
    errordlg('Cole lower bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.cole.ub) || isinf(par.cole.ub)
    errordlg('Cole upper bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.pearson.lb) || isinf(par.pearson.lb)
    errordlg('Pearson VII lower bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.pearson.ub) || isinf(par.pearson.ub)
    errordlg('Pearson VII upper bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.losev.lb) || isinf(par.losev.lb)
    errordlg('Losev lower bound must be a number','error','modal');
    err=1;
    return
end

if isnan(par.losev.ub) || isinf(par.losev.ub)
    errordlg('Losev upper bound must be a number','error','modal');
    err=1;
    return
end

% conditions: 
if par.delta.lb<-20
    errordlg('Pseudo Delta lower bound must be above -20','error','modal');
    err=1;
    return
end

if par.delta.ub>20
    errordlg('Pseudo Delta upper bound must be below 20','error','modal');
    err=1;
    return
end

if par.delta.ub<par.delta.lb
    errordlg('Pseudo Delta upper bound must be bigger the its lower bound','error','modal');
    err=1;
    return
end

if par.gauss.lb<-20
    errordlg('Gauss lower bound must be above -20','error','modal');
    err=1;
    return
end

if par.gauss.ub>20
    errordlg('Gauss upper bound must be below 20','error','modal');
    err=1;
    return
end

if par.gauss.ub<par.gauss.lb
    errordlg('Gauss upper bound must be bigger the its lower bound','error','modal');
    err=1;
    return
end

if par.lorenz.lb<-20
    errordlg('Lorentz lower bound must be above -20','error','modal');
    err=1;
    return
end

if par.lorenz.ub>20
    errordlg('Lorentz upper bound must be below 20','error','modal');
    err=1;
    return
end

if par.lorenz.ub<par.lorenz.lb
    errordlg('Lorentz upper bound must be bigger the its lower bound','error','modal');
    err=1;
    return
end

if par.sech.lb<-20
    errordlg('Sech lower bound must be above -20','error','modal');
    err=1;
    return
end

if par.sech.ub>20
    errordlg('Sech upper bound must be below 20','error','modal');
    err=1;
    return
end

if par.sech.ub<par.sech.lb
    errordlg('Sech upper bound must be bigger the its lower bound','error','modal');
    err=1;
    return
end

if par.cole.lb<-20
    errordlg('Cole lower bound must be above -20','error','modal');
    err=1;
    return
end

if par.cole.ub>20
    errordlg('Cole upper bound must be below 20','error','modal');
    err=1;
    return
end

if par.cole.ub<par.agauss.lb
    errordlg('Cole upper bound must be bigger the its lower bound','error','modal');
    err=1;
    return
end

if par.pearson.lb<-20
    errordlg('Pearson VII lower bound must be above -20','error','modal');
    err=1;
    return
end

if par.pearson.ub>20
    errordlg('Pearson VII upper bound must be below 20','error','modal');
    err=1;
    return
end

if par.pearson.ub<par.alorenz.lb
    errordlg('Pearson VII upper bound must be bigger the its lower bound','error','modal');
    err=1;
    return
end

if par.losev.lb<-20
    errordlg('Losev lower bound must be above -20','error','modal');
    err=1;
    return
end

if par.losev.ub>20
    errordlg('Losev upper bound must be below 20','error','modal');
    err=1;
    return
end

if par.losev.ub<par.asech.lb
    errordlg('Losev upper bound must be bigger the its lower bound','error','modal');
    err=1;
    return
end

% parameters conditions:
par.beta=str2double(get(handles.beta,'string'));
if isnan(par.beta) || isinf(par.beta)
    errordlg('Expected No. of Peaks must be a number.','error','modal');
    err=1;
    return
end

par.tau_diff=str2double(get(handles.tau_diff,'string'));
if isnan(par.tau_diff) || isinf(par.tau_diff)
    errordlg('The point difference needs to be a number.','error','modal');
    err=1;
    return
end

if par.beta<1
    errordlg('Expected No. of Peaks must be larger than 0.','error','modal');
    err=1;
    return
end

par.fitnessgauss=str2double(get(handles.fitnessgauss,'string'));
if isnan(par.fitnessgauss) || isinf(par.fitnessgauss)
    errordlg('The Normalization factor gauss must be a number between 0 and 0.3','error','modal');
    err=1;
    return
end

if par.fitnessgauss<0 || par.fitnessgauss>0.3
    errordlg('0<Normalization factor<0.3','error','modal');
    err=1;
    return
end

par.sigmamax=str2double(get(handles.width,'string'));
if isnan(par.sigmamax) || isinf(par.sigmamax)
    errordlg('The width factor must be a number.','error','modal');
    err=1;
    return
end

%if par.sigmamax<=par.sigmamin
%    errordlg(['The width factor must be a larger than ' num2str(par.sigmamin)] ,'error','modal');
%    err=1;
%    return
%end

par.alpha=str2double(get(handles.alpha,'string'));
if par.alpha<0.5 || par.alpha>0.9
    errordlg('0.5<Alpha<0.9','error','modal');
    err=1;
    return
end

if ~isfolder(dirname)
    errordlg('The directory specified does not exist.','Incorrect File Path','modal');
    return
end
save([dirname,'\','par.mat'],'par');

% Fitness & Complexity Button:
function Fitness(hObject,eventdata,handles) 
open('ISGP\complexity.pdf');

% Parameters defaults:
function par=ParDefaults()
global comm
load([comm.dirname,'\','data.GPDAT']); 
u=log10(1./(2.*pi.*data(:,1))); 
tau_max=ceil(u(1));
tau_min=-ceil(-u(end));

par.generations=300;
par.population=15;
par.runs=3;
par.mutation=0.4;
par.addpick=0.4;
par.remove=0.2;
par.stop=30;
par.duplicate=3;

par.delta.initial=1;
par.gauss.initial=0;
par.lorenz.initial=0;
par.sech.initial=0;
par.cole.initial=0;
par.pearson.initial=0;
par.losev.initial=0;
par.hn.initial=0;
par.kf.initial=0;
par.agauss.initial=0;
par.alorenz.initial=0;
par.asech.initial=0;
par.pseudovoigt.initial=0;
par.R.initial=0;
par.U.initial=0;

par.delta.mutation=1;
par.gauss.mutation=0;
par.lorenz.mutation=0;
par.sech.mutation=0;
par.cole.mutation=0;
par.pearson.mutation=0;
par.losev.mutation=0;
par.hn.mutation=0;
par.kf.mutation=0;
par.agauss.mutation=0;
par.alorenz.mutation=0;
par.asech.mutation=0;
par.pseudovoigt.mutation=0;
par.R.mutation=0;
par.U.mutation=0;

par.delta.lb=tau_min;
par.gauss.lb=tau_min;
par.lorenz.lb=tau_min;
par.sech.lb=tau_min;
par.cole.lb=tau_min;
par.pearson.lb=tau_min;
par.losev.lb=tau_min;
par.hn.lb=tau_min;
par.kf.lb=tau_min;
par.agauss.lb=tau_min;
par.alorenz.lb=tau_min;
par.asech.lb=tau_min;
par.pseudovoigt.lb=tau_min;
par.R.lb=-10;
par.U.lb=tau_max+1.5;

par.delta.ub=tau_max;
par.gauss.ub=tau_max;
par.lorenz.ub=tau_max;
par.sech.ub=tau_max;
par.cole.ub=tau_max;
par.pearson.ub=tau_max;
par.losev.ub=tau_max;
par.hn.ub=tau_max;
par.kf.ub=tau_max;
par.agauss.ub=tau_max;
par.alorenz.ub=tau_max;
par.asech.ub=tau_max;
par.pseudovoigt.ub=tau_max;
par.R.ub=-9;
par.U.ub=tau_max+3;

par.tau_diff=3;
par.beta=3;
par.lambda=0;
par.fitnessgauss=0.05;
par.max=1;
par.alpha=0.8;

par.sigmamax=-(u(end,1)-u(1,1));
%par.sigmamin=0.5*(u(1,1)-u(2,1));
par.forcedpick='';
par.forcedpickrange=[0,0];

% Load header:
function header(hObject,eventdata,handles) 
img=imread('ISGP/h3.isgp');
image(img);
axis off
axis image

% Load slogo:
%function slogo(hObject,eventdata,handles) 
%img=imread('ISGP/slg.isgp');
%image(img);
%axis off
%axis image

% set the remove Peak propability value:
function RemovePick(hObject,eventdata,handles) 
mut=str2double(get(handles.mutation,'String'));
add=str2double(get(handles.addpick,'String'));
remove=1-mut-add;
if remove<0.0001
    remove=0;
end
set(handles.remove,'String',num2str(remove));

% CheckBox CallBacks:
% Pseudo Delta:
function delta_check(hObject,eventdata,handles) 
if get(handles.delta.initial,'Value') || get(handles.delta.mutation,'Value')
    set(handles.delta.txt,'FontWeight','bold');
    set(handles.delta.lb,'Enable','on');
    set(handles.delta.ub,'Enable','on');
else
    set(handles.delta.txt,'FontWeight','normal');
    set(handles.delta.lb,'Enable','off');
    set(handles.delta.ub,'Enable','off');
end

% Gaussian:
function gauss_check(hObject,eventdata,handles)  
if get(handles.gauss.initial,'Value') || get(handles.gauss.mutation,'Value')
    set(handles.gauss.txt,'FontWeight','bold');
    set(handles.gauss.lb,'Enable','on');
    set(handles.gauss.ub,'Enable','on');
else
    set(handles.gauss.txt,'FontWeight','normal');
    set(handles.gauss.lb,'Enable','off');
    set(handles.gauss.ub,'Enable','off');
end

% Lorentzian:
function lorenz_check(hObject,eventdata,handles) 
if get(handles.lorenz.initial,'Value') || get(handles.lorenz.mutation,'Value')
    set(handles.lorenz.txt,'FontWeight','bold');
    set(handles.lorenz.lb,'Enable','on');
    set(handles.lorenz.ub,'Enable','on');
else
    set(handles.lorenz.txt,'FontWeight','normal');
    set(handles.lorenz.lb,'Enable','off');
    set(handles.lorenz.ub,'Enable','off');
end

% Sech:
function sech_check(hObject,eventdata,handles)  
if get(handles.sech.initial,'Value') || get(handles.sech.mutation,'Value')
    set(handles.sech.txt,'FontWeight','bold');
    set(handles.sech.lb,'Enable','on');
    set(handles.sech.ub,'Enable','on');
else
    set(handles.sech.txt,'FontWeight','normal');
    set(handles.sech.lb,'Enable','off');
    set(handles.sech.ub,'Enable','off');
end

% Cole-Cole:
function cole_check(hObject,eventdata,handles) 
if get(handles.cole.initial,'Value') || get(handles.cole.mutation,'Value')
    set(handles.cole.txt,'FontWeight','bold');
    set(handles.cole.lb,'Enable','on');
    set(handles.cole.ub,'Enable','on');
else
    set(handles.cole.txt,'FontWeight','normal');
    set(handles.cole.lb,'Enable','off');
    set(handles.cole.ub,'Enable','off');
end

% Pearson VII:
function pearson_check(hObject,eventdata,handles) 
if get(handles.pearson.initial,'Value') || get(handles.pearson.mutation,'Value')
    set(handles.pearson.txt,'FontWeight','bold');
    set(handles.pearson.lb,'Enable','on');
    set(handles.pearson.ub,'Enable','on');
else
    set(handles.pearson.txt,'FontWeight','normal');
    set(handles.pearson.lb,'Enable','off');
    set(handles.pearson.ub,'Enable','off');
end

% Losev:
function losev_check(hObject,eventdata,handles) 
if get(handles.losev.initial,'Value') || get(handles.losev.mutation,'Value')
    set(handles.losev.txt,'FontWeight','bold');
    set(handles.losev.lb,'Enable','on');
    set(handles.losev.ub,'Enable','on');
else
    set(handles.losev.txt,'FontWeight','normal');
    set(handles.losev.lb,'Enable','off');
    set(handles.losev.ub,'Enable','off');
end