function Run(varargin)
% Run(varargin) 
% Usage: This function runs the GP. Solve the IS mathematical model using a GP.
% varargin = {comm}
% The comm structure is the communication factor between the screens.
% comm.pos = The last figure position. The current figure will take its place.
% comm.dirname = The name of the work directory.
% comm.name = the project's name.
% comm.movie = Whether the "save movie" check box was checked (project).
% comm.plot = Whether to plot axexs on startup (LoadData)
% comm.UseFilter = Whether the filter was used the last time ().
% comm.LoadParamaters = Whether the user has changed any of the paramaters (Parameters).
% comm.path1 = The Path to DataSet1.
% comm.path2 = The Path to DataSet2.
% comm.spath1 = The short Path name to DataSet1.
% comm.spath2 = The short Path name to DataSet2.
% NOTE:
% comm.dirname is the WORKING dir name.
% comm.path# is the DATA dir name.
% Credits: Alex Tesler, Yehu David-Horev, Alon Dana, Maor Asher.
% Modified by: Arthur Doroshev
% Pause indicates whether ISGP is paused.
% codegen
%--------------------------------------------------------------------------
global Pause comm par h m s screen5 pos count_run omega_duplicate
Pause=0;
h=0;
m=0;
s=0;
if nargin==0    
    % Set Default values:
    comm.pos=[240,227,800,550];
    comm.ramp(1:2,1:5)=-1;
    comm.dirname='0';
    comm.existingfile='';
    comm.name='';
    comm.movie=0;
    comm.plot=0;
    comm.UseFilter=0;
    comm.LoadParamaters=0;
    comm.path1='';
    comm.path2='';
    comm.spath1='';
    comm.spath2='';
    comm.spath3='';
    comm.spath4='';
    comm.kktcheckbox=0;
else
    comm=varargin{1};
end

comm.Abort=0;

%Set pos to [140,227,1000,550]:
pos=[140,227,1000,670];
screen5=figure('Position',pos,'Color','White','MenuBar','none','NumberTitle','off','Name','Impedance Spectroscopy Genetic Programming','Resize','off','Tag','screen5');
%if comm.movie==1
    %set(screen5,'WindowStyle','modal');
%end
movegui(screen5,'center')
handles=guihandles;
guidata(screen5,handles);
load([comm.dirname,'\','par.mat']);

%------------ Buttons: ------------%
% Close:
handles.close=uicontrol('Style','pushbutton','Units','pixels','Parent',screen5,'Position',[10,120,70,25],'FontSize',8,'FontWeight','bold','Enable','off','String','Close');

% Help:
handles.help=uicontrol('Style','pushbutton','Units','pixels','Parent',screen5,'Position',[100,120,70,25],'FontSize',8,'FontWeight','bold','String','Help');

% Finish:
handles.finish=uicontrol('Style','pushbutton','Units','pixels','Parent',screen5,'Position',[920,120,70,25],'FontSize',8,'FontWeight','bold','Enable','off','String','Finish');

% Back:
handles.back=uicontrol('Style','pushbutton','Units','pixels','Parent',screen5,'Position',[830,120,70,25],'FontSize',8,'FontWeight','bold','Enable','off','String','<  Back');

% Abort:
handles.abort=uicontrol('Style','pushbutton','Units','pixels','Parent',screen5,'Position',[740,120,70,25],'FontSize',8,'FontWeight','bold','Enable','on','String','Abort');

% Pause:
handles.pause=uicontrol('Style','pushbutton','Units','pixels','Parent',screen5,'Position',[650,120,70,25],'FontSize',8,'FontWeight','bold','Enable','on','String','Pause');

% Kill Run:
handles.killrun=uicontrol('Style','pushbutton','Units','pixels','Parent',screen5,'Position',[560,120,70,25],'FontSize',8,'FontWeight','bold','Enable','on','String','Kill Run');

% slogo:
axes('Units','pixels','Position',[0,628,118,37],'CreateFcn',{@slogo,handles});

% text: Project Name
handles.proj=uicontrol('Style','text','Units','pixels','Parent', screen5,'Position',[845,645,45,20],'BackgroundColor','White','FontSize',9,'HorizontalAlignment','left','String','Project: ');
handles.projname=uicontrol('Style','text','Units','pixels','Position',[890,645,110,20],'BackgroundColor','White','FontSize',9,'FontWeight','bold','HorizontalAlignment','left','String',comm.name);

% text: "Project Aborted!"
handles.projectaborted=uicontrol('Style','text','Units','pixels','Position',[845,645,100,20],'Visible','off','BackgroundColor','White','ForegroundColor','Red','FontSize',10,'HorizontalAlignment','left','String','Project Aborted!');

if comm.dirname=='0'
    set(handles.projname,'String','No Project Loaded');
    set(handles.projname,'ForegroundColor','red');
    set(handles.projname,'FontAngle','italic');
    set(handles.projname,'FontWeight','normal');
end

%------------ Axes: ------------%
handles.ax1=axes('Units','pixels','Position',[60,430,400,190],'Parent',screen5);
xlabel('Z','fontsize',10)
ylabel('-Z"','fontsize',10)

handles.ax2=axes('Units','pixels','Position',[560,430,400,190],'Parent',screen5);
xlabel('log_1_0(\tau)','fontsize',10)
ylabel('\Gamma','fontsize',10)

handles.ax3=axes('Units','pixels','Position',[560,190,400,190],'Parent',screen5);
xlabel('Complexity','fontsize',10)
ylabel('Discrepancy','fontsize',10)

handles.ax4=axes('Units', 'pixels','Position',[60,190,400,190],'Parent',screen5);
xlabel('\omega','fontsize',10)
ylabel('Residual','fontsize',10)

%------------ uicontrols: ------------%
% text: "Current Run:"
uicontrol('Style','text','Units','pixels','Parent', screen5,'Position',[15,85,140,20],'BackgroundColor','White','FontSize',12,'HorizontalAlignment','left','String','Current Run:');
handles.run=uicontrol('Style','text','Units','pixels','Enable','inactive','Parent', screen5,'Position',[140,85,30,20],'BackgroundColor','White','FontSize',10,'FontWeight','bold','HorizontalAlignment','center','String','0');
uicontrol('Style','text','Units','pixels','Enable','inactive','Parent', screen5,'Position',[165,85,30,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',[' \',num2str(par.runs)]);

% text: "Generation":
uicontrol('Style','text','Units','pixels','Parent', screen5,'Position',[15,60,140,20],'BackgroundColor','White','FontSize',12,'HorizontalAlignment','left','String','Generation:');
handles.generation=uicontrol('Style','text','Units','pixels','Enable','inactive','Parent', screen5,'Position',[140,60,30,20],'BackgroundColor','White','FontSize',10,'FontWeight','bold','HorizontalAlignment','center','String','0');
uicontrol('Style','text','Units','pixels','Enable','inactive','Parent',screen5,'Position',[165,60,60,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String',[' \',num2str(par.generations)]);

% text: "Compatibility :"
uicontrol('Style','text','Units','pixels','Parent',screen5,'Position',[15,35,140,20],'BackgroundColor','White','FontSize',12,'HorizontalAlignment','left','String','Compatibility :');
handles.compatibility=uicontrol('Style','text','Units','pixels','Enable','inactive','Parent', screen5,'Position',[140,35,100,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String','0');

% text: "Area :"
uicontrol('Style','text','Units','pixels','Parent',screen5,'Position',[15,10,140,20],'BackgroundColor','White','FontSize',12,'HorizontalAlignment','left','String','Area :');
handles.Area=uicontrol('Style','text','Units','pixels','Enable','inactive','Parent',screen5,'Position',[140,10,100,20],'BackgroundColor','White','FontSize',10,'HorizontalAlignment','center','String','0');

% text: "Best Model ":
uicontrol('Style','text','Units','pixels','Parent',screen5,'Position',[520,60,140,20],'BackgroundColor','White','FontSize',12,'HorizontalAlignment','left','String','Best Model :');
handles.bestmodel=uicontrol('Style','text','Units','pixels','HorizontalAlignment','Left','Parent',screen5,'Position',[645,18,350,60],'BackgroundColor','White','FontSize',10,'String','');

% text: "Elapsed time :"
uicontrol('Style','text','Units','pixels','Parent',screen5,'Position',[520,85,140,20],'BackgroundColor','White','FontSize',12,'HorizontalAlignment','left','String','Elapsed time :');
handles.time=uicontrol('Style','text','Units','pixels','Parent',screen5,'Position',[645,85,140,20],'BackgroundColor','White','ForegroundColor','red','FontSize',12,'HorizontalAlignment','left','String','00:00:00');

% 1Define Buttons' Callbacks:
set(handles.close,'CallBack',{@PBClose,handles});
set(handles.help,'CallBack',{@PBHelp,handles});
set(handles.finish,'CallBack',{@PBFinish,handles,comm});
set(handles.back,'CallBack',{@PBBack,handles,comm});
set(handles.abort,'CallBack',{@PBAbort,handles});
set(handles.pause,'CallBack',{@PBPause,handles});
set(handles.killrun,'CallBack',{@PBKillRun,handles});

% TIMER:
handles.t=timer('TimerFcn',{@elapsed_time,handles},'Period',1.0,'ExecutionMode','fixedDelay','ObjectVisibility','off');
start(handles.t);

%=========================================================================%
%%% ALGORITHEM START %%%
%=========================================================================%
% making a movie => as presented
% the movie is of the evolving func. (col.col diagram)
makemovie=comm.movie;
t=clock; % clock shows the time
% starts a "lap time" for the run
clear mex  
% running "par.runs" times
for count_run=1:par.runs
    % creates the output text file
    dlmwrite([comm.dirname,'\output\results_',num2str(count_run),'.txt'],['ISGP project:' comm.name],'delimiter','','newline','pc');
    dlmwrite([comm.dirname,'\output\results_',num2str(count_run),'.txt'],'-------------------------------------------------------------------------------','-append','delimiter','','newline','pc');
    dlmwrite([comm.dirname,'\output\results_',num2str(count_run),'.txt'],['Run number:',num2str(count_run)],'-append','delimiter','','newline','pc');
    dlmwrite([comm.dirname,'\output\results_',num2str(count_run),'.txt'],'-------------------------------------------------------------------------------','-append','delimiter','','newline','pc');
    set(handles.run,'String',num2str(count_run));
    % 1data upload
    load([comm.dirname,'\','data.GPDAT']); 
    load([comm.dirname,'\','data_val.GPDAT']);
    [rr,cc]=size(data); 
    % Set up the input (u) and output (y) variables.
    u=log10(1./(2.*pi.*data(:,1)));      % transfer the frequency to
    uv=log10(1./(2.*pi.*data_val(:,1))); % logarithmic relaxation time
    y=data(:,3);
    yv=data_val(:,3); %Im
    yr=data(:,2); 
    yrv=data_val(:,2); %Re
    % if you wanted a movie
    if makemovie==1
        mov=VideoWriter([comm.dirname,'\output\',comm.name,'_',num2str(count_run)]); 
        mov.FrameRate=1;
        frmcnt=1;
    end
    % start the run! work!
    gp_ini;
    % changed again in order to save the run
    if makemovie==1
        try
            figure(screen5);
            frm(frmcnt)=getframe(gcf);
            frmcnt=frmcnt+1;
            open(mov);
            for fcount=1:length(frm)
            writeVideo(mov,frm(fcount));
            end
            close(mov);
        catch
            figure(screen5);
            set(screen5,'Position',pos);
            frm(frmcnt)=getframe(gcf);
            frmcnt=frmcnt+1;
            open(mov);
            for fcount=1:length(frm)
            writeVideo(mov,frm(fcount));
            end
            close(mov);
        end
    end
    % ends the text file
    dlmwrite([comm.dirname,'\output\results_',num2str(count_run),'.txt'],'-------------------------------------------------------------------------------','-append','delimiter','','newline','pc');
    dlmwrite([comm.dirname,'\output\results_',num2str(count_run),'.txt'],['Elapsed Time:',get(handles.time,'String')],'-append','delimiter','','newline','pc');
    dlmwrite([comm.dirname,'\output\results_',num2str(count_run),'.txt'],'-------------------------------------------------------------------------------','-append','delimiter','','newline','pc');
    % create final result picture
    set(screen5,'WindowStyle','modal');
    figure(screen5);
    set(screen5,'Position',pos);
    a=getframe(gcf);
    imwrite(a.cdata,[comm.dirname,'\output\final result_',num2str(count_run),'.jpg'])
    set(screen5,'WindowStyle','normal');
end

if any(~strcmp(comm.path3,''))
    if ~strcmp(char(comm.path3(1)),comm.path1)
        comm.spath1=char(comm.spath3(1));
        comm.spath2=char(comm.spath4(1));
        comm.path1=char(comm.path3(1));
        comm.path2=char(comm.path4(1));
        omega_duplicate=load([comm.dirname,'\','omega.gpdat'],'-ascii');
        comm.dirname=char(comm.dirname2(1));
        comm.name=comm.dirname;
        comm.check=1;
        for i=2:length(comm.path3)
            if strcmp(comm.path3{i},'')
                continue
            end
            comm.spath3(1:(end-i+1))=comm.spath3(i:end);
            comm.spath3(end-i+2:end)='';
            comm.path3(1:(end-i+1))=comm.path3(i:end);
            comm.path3(end-i+2:end)='';
            comm.spath4(1:(end-i+1))=comm.spath4(i:end);
            comm.spath4(end-i+2:end)='';
            comm.path4(1:(end-i+1))=comm.path4(i:end);
            comm.path4(end-i+2:end)='';
            comm.dirname2(1:(end-i+1))=comm.dirname2(i:end);
            comm.dirname2(end-i+2:end)='';
            break
        end
    else
        comm.check=0;
    end
else
    comm.check=0;
end

if comm.check==1
    stop(handles.t);
    delete(handles.t);
    delete(handles.screen5);
    project(comm);
end  

% close
% End time
set(handles.finish,'Enable','on');
set(handles.abort,'Enable','off');
set(handles.back,'Enable','off');
set(handles.pause,'Enable','off');
set(handles.killrun,'Enable','off');
stop(handles.t);
delete(handles.t);
if comm.movie==1
    set(screen5,'WindowStyle','normal');
end
handles.Pausedlg=figure('Position',[500,500,400,250],'Color','White','WindowStyle','modal','MenuBar','none','NumberTitle','off','Name','ISGP - Done','Resize','off','Tag','Pausedlg');
movegui(handles.Pausedlg,'center');

% EndPic:
axes('Units','pixels','Parent',handles.Pausedlg,'Position',[5,20,220,210],'CreateFcn',{@EndPic,handles});

% EndTxt:
uicontrol('Style','text','Units','pixels','Position',[250,35,130,200],'BackgroundColor','White','FontSize',30,'HorizontalAlignment','center','FontWeight','bold','String','The Work is Done!');

% Continue:
handles.continue=uicontrol('Style','pushbutton','Units','pixels','Position',[275,10,70,25],'FontSize',8,'FontWeight','bold','Enable','on','CallBack',{@PBContinue,handles},'String','Continue');
uiwait(handles.Pausedlg);

% run=1;
% set(handles.run,'String',num2str(run));

% **
% The user's input should be addressed as follows:
% **

% comm.movie
% Whether to save the movie. (1/0)[comm stands for communication, as in between functions/screens...
% This screen receives "comm" through varargin]

% comm.dirname
% the working directory for loading files such as data.gpdat, data_val.gpdat...

% comm.existingfile
% if you work with an existing file this is its directory

% comm.UseFilter
% Whether the user wants a filter

% omega.gpdat
% the FILE that stores w0, w1 for the filter.(Also located in "comm.dirname")

% par.mat
% the FILE that stores the parameters. (Also located in "comm.dirname")
% Its variables are:

% par.generations                           % # of generations
% par.population                            % population size = # of function inside the set
% par.runs                                  % # of runs
% par.mutation                              % mutation propability
% par.addpick                               % add pick propability
% par.remove                                % remove pick propability

% par.gauss.initial                         % Whether the user wants GAUSS in the initial population (1/0)
% par.lorenz.initial                        % "
% par.sech.initial                          % "
% par.pearson.initial                       % "
% par.pseudovoigt.initial                   % "
% par.agauss.initial                        % "
% par.alorenz.initial                       % "
% par.asech.initial                         % "

% par.gauss.mutation                        % Whether the user wants GAUSS in the  mutations (1/0)
% par.lorenz.mutation                       % "
% par.sech.mutation                         % "
% par.pearson.mutation                      % "
% par.pseudovoigt.mutation                  % "
% par.ungauss.mutation                      % "
% par.unlorenz.mutation                     % "
% par.unsech.mutation                       % "

% par.beta                                  % advanced paramater for the fitness = guessed # of peaks
% par.lambda                                % advanced paramater for the fitness = actual # of peaks
% par.fitnessgauss                          % advanced paramater for the fitness = normalization factor, max of 0.3
% par.max                                   % maximum descendants per individual function
% pa.sigmamax                               % in the calculation for the width sigmoid, where the sigmoid equals to 0.5 = -(tau(length(tau(:,1)),1)-tau(1,1))
% par.forcedpick                            % the forced pick the user choose, for example 'G'
% par.forcedpickrange                       % the range of the forced pick, for example [1 2]
% par.duplicate                             % # of times the algorithms runs threw the set of function before it continues
% par.stop                                  % the run stopes after it recieves that # of same result

%=========================================================================%
%%% ALGORITHEM END %%%
%=========================================================================%

%------------ Functions: ------------%
% Create
%function CreateFunction()
%clc

% Close Button
function PBClose(hObject,eventdata,handles)
button=questdlg('Are you sure you want to close the ISGP?','Close Confirmation Dialog','Yes','No','Yes');
if strcmp(button,'Yes')
    %if ~isfolder(comm.dirname)
    %h=errordlg('The directory specified does not exist.','Incorrect File Path');
    %uiwait(h)
    %else
    %save([comm.dirname,'\','comm.mat'] ,'comm');
    %end
    delete(handles.screen5)
end

% Help Button:
function PBHelp(hObject,eventdata,handles) 
web http://electroceramics.technion.ac.il/isgp/help.html

% Finish Button:
function PBFinish(hObject,eventdata,handles,comm) 
comm.pos=get(handles.screen5,'Position');
winopen([comm.dirname '\output'])
delete(handles.screen5)

% Back Button:
function PBBack(hObject,eventdata,handles,comm) 
Parameters(comm)
delete(handles.screen5)

% Abort Button:
function PBAbort(hObject,eventdata,handles) 
global Pause mov makemovie comm screen5 pos count_run
if comm.movie==1
       set(screen5,'WindowStyle','normal');
% if ISGP ia already peused, open the pause window:
if Pause==1
    figure(2)
    return
end
button=questdlg('Are you sure you want to abort the run?','Abort Confirmation Dialog','Yes','No','No');
if strcmp(button,'Yes')
    set(handles.back,'Enable','on')
    set(handles.close,'Enable','on')
    set(handles.proj,'Visible','off')
    set(handles.projname,'Visible','off')
    set(handles.projectaborted,'Visible','on')
    set(handles.abort,'Enable','off')
    set(handles.pause,'Enable','off')
    if makemovie==1
        try
            figure(screen5);
            frm(frmcnt)=getframe(figure(screen5));  
            frmcnt=frmcnt+1;
            %mov=addframe(mov,frm);
            open(mov);
            for fcount=1:length(frm)
            writeVideo(mov, frm(fcount));
            end
            close(mov);
            %mov=close(mov);
        catch
            figure(screen5);
            set(screen5,'Position',pos);
            frm(frmcnt)=getframe(figure(screen5));
            frmcnt=frmcnt+1; 
            %mov=addframe(mov,frm);
            %mov=close(mov);
            open(mov);
            for fcount=1:length(frm)
            writeVideo(mov, frm(fcount));
            end
            close(mov);
        end
    end

    dlmwrite([comm.dirname,'\output\results_',num2str(count_run),'.txt'],'-------------------------------------------------------------------------------','-append','delimiter','','newline','pc');
    dlmwrite([comm.dirname,'\output\results_',num2str(count_run),'.txt'],'PROJECT ABORTED!!!','-append','delimiter','','newline','pc');
    comm.Abort=1;
else
    if comm.movie==1
        set(screen5,'WindowStyle','modal');
    end
    return
end
end

% Kill Run Button:
function PBKillRun(hObject,eventdata,handles) 
global Pause mov makemovie comm screen5 pos count_run
comm.check=0;
comm.path3='';
if comm.movie==1
    set(screen5,'WindowStyle','normal');
end

% If ISGP ia already paused, open the pause window:
if Pause==1
    figure(2)
    return
end

button=questdlg('Are you sure you want to abort the run?','Abort Confirmation Dialog','Yes','No','No');
if strcmp(button,'Yes')
    set(handles.back,'Enable','on');
    set(handles.close,'Enable','on');
    set(handles.proj,'Visible','off');
    set(handles.projname,'Visible','off');
    set(handles.projectaborted,'Visible','on');
    set(handles.abort,'Enable','off');
    set(handles.pause,'Enable','off');
    if makemovie==1
        try
            figure(screen5);
            frm(frmcnt)=getframe(figure(screen5)); 
            frmcnt=frmcnt+1;
            %mov=addframe(mov,frm);
            open(mov);
            for fcount=1:length(frm)
               writeVideo(mov,frm(fcount));
            end
            close(mov);
            %mov=close(mov);
        catch
            figure(screen5);
            set(screen5,'Position',pos);
            frm(frmcnt)=getframe(figure(screen5));
            frmcnt=frmcnt+1; 
            open(mov);
            for fcount=1:length(frm)
               writeVideo(mov,frm(fcount));
            end
            close(mov);
        end
    end
    dlmwrite([comm.dirname,'\output\results_',num2str(count_run),'.txt'],'-------------------------------------------------------------------------------','-append','delimiter','','newline','pc');
    dlmwrite([comm.dirname '\output\results_' num2str(count_run) '.txt'],'PROJECT ABORTED!!!' ,'-append','delimiter','','newline','pc');
    comm.Abort=1;
else
    if comm.movie==1
        set(screen5,'WindowStyle','modal');
    end
    return
end

% Pause Button:
function PBPause(hObject,eventdata,handles) 
global Pause screen5 comm
if comm.movie==1
        set(screen5,'WindowStyle','normal');
end

% If ISGP ia already paused, open the pause window:
if Pause==1
    figure(2)
    return
end
Pause=1;

% Open new window:
handles.Pausedlg=figure('Position',[500,500,250,130],'Color','White','MenuBar','none','NumberTitle','off','Name','ISGP - Paused','Resize','off','Tag','Pausedlg');
movegui(handles.Pausedlg,'center')

% PausePic:
axes('Units','pixels','Position',[0,0,109,128],'CreateFcn',{@PausePic,handles});

% PauseTxt:
axes('Units','pixels','Position',[130,50,94,66],'CreateFcn',{@Paused,handles});

% Continue:
handles.continue=uicontrol('Style','pushbutton','Units','pixels','Position',[90,10,70,25],'FontWeight','bold','Enable','on','CallBack',{@PBContinue,handles},'String','Continue');
uiwait(handles.Pausedlg)
Pause=0;

% Continue Button
function PBContinue(hObject,eventdata,handles) 
delete(handles.Pausedlg)

% Load header:
%function header(hObject,eventdata,handles)
%img=imread('ISGP/h3.isgp');
%image(img);
%axis off
%axis image

% Load slogo:
function slogo(hObject,eventdata,handles) 
img=imread('ISGP/slg.isgp');
image(img);
axis off
axis image

% Load PausePic:
function PausePic(hObject,eventdata,handles) 
img=imread('ISGP/pause.isgp');
image(img);
axis off
axis image

% Load EndPic:
function EndPic(hObject,eventdata,handles) 
img=imread('ISGP/SF.isgp');
image(img);
axis off
axis image

% Load PauseTxt:
function Paused(hObject,eventdata,handles)
img=imread('ISGP/paused.isgp');
image(img);
axis off
axis image

% Elapsed time:
function elapsed_time(hObject,eventdata,handles)
global h m s
s=s+1;
if s==60
    m=m+1;
    s=0;
end
if m==60
    h=h+1;
    m=0;
end
if h<10
    hour=['0',num2str(h)];
else
    hour=num2str(h);
end
if m<10
    min=['0',num2str(m)];
else
    min=num2str(m);
end
if s<10
    sec=['0',num2str(s)];
else
    sec=num2str(s);
end
set(handles.time,'String',[hour,' : ',min,' : ',sec]);