function plot_kkt2(varargin)
% plot_kkt2(varargin)
% Usage: This function plots the KKT result.
% Written by: Maor Asher
% Modified by: Arthur Doroshev
%--------------------------------------------------------------------------
global no_kkt comm
KKT1=figure('Position',[235,157,800,580],'Color','White','MenuBar','none','NumberTitle','off','Name','Impedance Spectroscopy Genetic Programming','Resize','off','Tag','KKT1');
if nargin==0
    % Set Default values:
    comm.pos=[230,217,800,550];
    comm.dirname='0';
    comm.name='';
    comm.existingfile='';
    comm.ramp(1:2,1:5)=-1;
    comm.movie=0;
    comm.plot=0;
    comm.UseFilter=0;
    comm.LoadParamaters=0;
    comm.path1='';
    comm.path2='';
    comm.spath1='';
    comm.spath2='';
else
    comm=varargin{1};
    llsl=varargin{2};
end

if nargin==0
    movegui(KKT1,'center');
end

handles=guihandles;
guidata(KKT1,handles);

%------------ Buttons: ------------%
KKTData=uipanel('Title','KKT Data Set 2','FontSize',10,'Units','pixels','Parent',KKT1,'Position',[2 2 795 545],'BackgroundColor','White');

% Close:
handles.close=uicontrol('Style','pushbutton','Units','pixels','Parent',KKTData,'Position',[720 10 70 25],'FontWeight','bold','String','Close');

% slogo:
axes('Units','pixels','Parent',KKT1,'Position',[326,543,118,37],'CreateFcn',{@slogo,handles});

% text: Project Name
uicontrol('Style','text','Units','pixels','Parent',KKT1,'Position',[645,560,45,20],'BackgroundColor','White','FontSize',9,'HorizontalAlignment','left','String','Project: ');
n=uicontrol('Style','text','Units','pixels','Parent',KKT1,'Position',[690,560,110,20],'BackgroundColor','White','FontSize',9,'FontWeight','bold','HorizontalAlignment','left','String',comm.name);

if comm.dirname=='0'
    set(n,'String','No Project Loaded');
    set(n,'ForegroundColor','red');
    set(n,'FontAngle','italic');
    set(n,'FontWeight','normal');
end

%------------ Axes: ------------%
handles.ax1=axes('Units','pixels','Parent',KKTData,'Position',[100,400,600,110],'Visible','on');
handles.ax2=axes('Units','pixels','Parent',KKTData,'Position',[100,220,600,110],'Visible','on');
handles.ax3=axes('Units','pixels','Parent',KKTData,'Position',[100,40,400,110],'Visible','on');

data=load([comm.dirname,'\','data_val.gpdat'],'-ascii');
f=data(:,1);
Re_z=no_kkt.data_val(:,2);
Im_z=no_kkt.data_val(:,3);

%Re_z=Re_z(:);
%Im_z=Im_z(:);
%f=f(:);
w=2*pi*f; % was w1
%a=1;
%w=eval(['w' num2str(a)]);
i=sqrt(-1);
Z=Re_z-i*Im_z;
ReZ=Re_z;
ImZ=Im_z;
index=length(w);

% Real from imaginary part:
ReKK=zeros(index,1);
for m=1:index
    for k=1:index
        if k~=m
            RKK(k)=(w(k)*imag(Z(k))-w(m)*imag(Z(m)))/((w(k))^2-w(m)^2); 
        end
        if k==m && k~=1 && k~=index
            RKK(k)=0.5*(imag(Z(k))/w(k)+((imag(Z(k+1))-imag(Z(k-1)))/(w(k+1)-w(k-1))));
        end
        if k==m && k==1 && k~=index
            RKK(k)=0.5*(imag(Z(k))/w(k)+((imag(Z(k+1))-imag(Z(k)))/(w(k+1)-w(k))));
        end
        if k==m && k~=1 && k==index
            RKK(k)=0.5*(imag(Z(k))/w(k)+((imag(Z(k))-imag(Z(k-1)))/(w(k)-w(k-1))));
        end
    end
    [pp,p]=csaps(w,RKK); 
    Re_KK1=diff(fnval(fnint(pp),[w(1),w(end)]));

    RKK=RKK(:);
    ReKK=0;
    trap=0; 
    dw=0; 
    for mm=1:index-1
        trap=(RKK(mm+1)+RKK(mm))/2;
        dw=w(mm+1)-w(mm);
        ReKK=ReKK+trap*dw;
    end
    Real_KK(m)=(2/pi)*ReKK+Re_z(end); 
    Real_KK1(m)=(2/pi)*Re_KK1+Re_z(end); 
end

% Imaginary from Real part:
ImKK=zeros(index,1); 
for m=1:index
    for k=1:index
        if k~=m
            IKK(k)=(real(Z(k))-real(Z(m)))/((w(k))^2-w(m)^2); 
        end
        if k==m && k~=1 && k~=index
            IKK(k)=(((real(Z(k+1))-real(Z(k-1)))/(w(k+1)-w(k-1))))/(2*w(k));
        end
        if k==m && k==1 && k~=index
            IKK(k)=(((real(Z(k+1))-real(Z(k)))/(w(k+1)-w(k))))/(2*w(k));
        end
        if k==m && k~=1 && k==index
            IKK(k)=((real(Z(k))-real(Z(k-1)))/(w(k)-w(k-1)))/(2*w(k)); 
        end
    end
    [pp,p]=csaps(w,IKK); 
    Im_KK1=diff(fnval(fnint(pp),[w(1),w(end)]));
    
    ImKK=0;
    trap=0; 
    dw=0; 
    for mm=1:index-1
        trap=(IKK(mm+1)+IKK(mm))/2;
        dw=w(mm+1)-w(mm);
        ImKK=ImKK+trap*dw;
    end
    Imag_KK(m)=(2*w(m)/pi)*(ImKK); 
    Imag_KK1(m)=(2*w(m)/pi)*(Im_KK1); 
end

% Accuracy of calculations:
Im_ex_max=max(abs(imag(Z)));
Re_ex_max=max(abs(real(Z)));
AER=(100/Re_ex_max/index)*sum(abs(ReZ(:)-Real_KK(:))); 
AEI=(100/Im_ex_max/index)*sum(abs(ImZ(:)-Imag_KK(:))); 
AER1=(100/(Re_ex_max*index))*sum(abs(ReZ(:)-Real_KK1(:))); 
AEI1=(100/(Im_ex_max*index))*sum(abs(ImZ(:)-Imag_KK1(:)));

%fprintf('Average Error of Real part (trapezoidal) [percent]: %.4g \n',AER);
%fprintf('Average Error of Imag part (trapezoidal) [percent]: %.4g \n',AEI);
fprintf('Average Error of Real part [percent]: %.4g \n',AER1);
fprintf('Average Error of Imag part [percent]: %.4g \n',AEI1);


% Polarization Resistance indirect calculations:
RpKK=imag(Z)./w;
Rp_KK=0;
trap=0; 
dw=0; 
for mm=1:index-1
    trap=(RpKK(mm+1)+RpKK(mm))/2;
    dw=w(mm+1)-w(mm);
    Rp_KK=Rp_KK+trap*dw;
end
Rp_KK=(2/pi)*Rp_KK;

% Polarization Resistance direct calculation:
Rpd=real(Z(1))-real(Z(end));
AERp=abs(abs(Rp_KK)-abs(Rpd))/abs(Rpd)*100; 

fprintf('Average Error of Polarization resistance [percent]: %.4g \n',AERp);

% R^2 calculation
SSR_Re=sum((Real_KK1-mean(Re_z)).^2); % sum squared regression 
SSR_Im=sum((Imag_KK1-mean(Im_z)).^2); % sum squared regression
SSE_Re=sum((Real_KK1(:)-Re_z(:)).^2); % sum squared errors 
SSE_Im=sum((Imag_KK1(:)-Im_z(:)).^2); % sum squared errors
SST_Re=sum((Re_z-mean(Re_z)).^2); % total sum of squares
SST_Im=sum((Im_z-mean(Im_z)).^2); % total sum of squares
Re_R=SSR_Re/SST_Re; % R^2 of Real part
Im_R=SSR_Im/SST_Im; % R^2 of Imag part

fprintf('R^2 of Real part: %.4g \n',Re_R);
fprintf('R^2 of Imag part: %.4g \n',Im_R);

warning off

if llsl==2
    semilogx(handles.ax1,w,Real_KK,w,Re_z,'.r');
    title(handles.ax1,'Kramers-Kronig Relation Imaginary to Real');
    xlabel(handles.ax1,'\omega');
    ylabel(handles.ax1,'Z''');
    legend(handles.ax1,'Kramers-Kronig Relation','Measured Data','Location','EastOutside');

    semilogx(handles.ax2,w,-Imag_KK,w,-Im_z,'.r');
    title(handles.ax2,'Kramers-Kronig Relation Real to Imaginary')
    xlabel(handles.ax2,'\omega');
    ylabel(handles.ax2,'-Z''''');
    legend(handles.ax2,'Kramers-Kronig Relation','Measured Data','Location','EastOutside');

    plot(handles.ax3,Real_KK,-Imag_KK,Re_z,-Im_z,'.r');
    title(handles.ax3,'Cole-Cole Plot');
    xlabel(handles.ax3,'Z''');
    ylabel(handles.ax3,'-Z''''');

else

    loglog(handles.ax1,w,Real_KK,w,Re_z,'.r');
    title(handles.ax1,'Kramers-Kronig Relation Imaginary to Real');
    xlabel(handles.ax1,'\omega');
    ylabel(handles.ax1,'Z''');
    legend(handles.ax1,'Kramers-Kronig Relation','Measured Data','Location','EastOutside');

    loglog(handles.ax2,w,-Imag_KK,w,-Im_z,'.r');
    title(handles.ax2,'Kramers-Kronig Relation Real to Imaginary')
    xlabel(handles.ax2,'\omega');
    ylabel(handles.ax2,'-Z''''');
    legend(handles.ax2,'Kramers-Kronig Relation','Measured Data','Location','EastOutside');

    plot(handles.ax3,Real_KK,-Imag_KK,Re_z,-Im_z,'.r');
    title(handles.ax3,'Cole-Cole Plot');
    xlabel(handles.ax3,'Z''');
    ylabel(handles.ax3,'-Z''''');
end

set(handles.close,'CallBack',{@PBClose,handles});
uicontrol('Style','text','Units','pixels','Parent', KKT1,'Position',[520,160,250,30],'BackgroundColor','White','FontSize',12,'HorizontalAlignment','left','String','Z_real(infinity) from K-K is: ');
uicontrol('Style','text','Units','pixels','Parent', KKT1,'Position',[520,120,250,30],'BackgroundColor','White','FontSize',12,'HorizontalAlignment','left','String',num2str(Re_z(end)));

%------------ Functions: ------------%
% Create
%function CreateFunction()
%clc

% Close Button:
function PBClose(hObject,eventdata,handles) 
delete(handles.KKT1)

% Load slogo:
function slogo(hObject,eventdata,handles) 
img=imread('ISGP/slg.isgp');
image(img);
axis off
axis image